@@ -1,34 +1,63 @@
 """Wall Pad Socket Client"""
 import asyncio
-
 import logging
+import time
 from typing import Any
 
 _CONNECTION_TIMEOUT = 5.0
 _READ_TIMEOUT = 5.0
 _QUEUE_WAIT_TIMEOUT = 1.0
 _DEFAULT_READ_BUFFER = 512  # 512 bytes
+_READ_WRITE_INTERVAL = 0.3
+_WRITE_RETRY_COUNT = 5
 
 _LOGGER = logging.getLogger(__name__)
 
 
+class _WpSendData:
+    """Socket send data"""
+
+    def __init__(self, packet: bytes):
+        self._complete = asyncio.Event()
+        self._try_cnt = 0
+        self._result = False
+        self.packet = packet
+
+    def inc_try(self) -> int:
+        """Socket send try"""
+        self._try_cnt += 1
+        return self._try_cnt
+
+    def set(self, result: bool) -> None:
+        """Socket send result"""
+        self._result = result
+        if not self._complete.is_set():
+            self._complete.set()
+
+    async def wait(self) -> bool:
+        """Socket send wait"""
+        await self._complete.wait()
+        return self._result
+
+
 class WpSocketClient:
     """Wall Pad Socket Client"""
     _reader: asyncio.StreamReader
     _writer: asyncio.StreamWriter
 
     def __init__(self, host: str, port: int, async_packets_handler):
-        self._host = host
-        self._port = port
+        self.host = host
+        self.port = port
         self._wait_tasks: Any = None
         self._async_receive_handler = async_packets_handler
 
         self._loop = asyncio.get_event_loop()
         self._receive_packet_queue: asyncio.Queue[bytes] = asyncio.Queue()
-        self._send_packet_queue: asyncio.Queue[bytes] = asyncio.Queue()
+        self._send_packet_queue: asyncio.Queue[_WpSendData] = asyncio.Queue()
 
-        self._retry_cnt = 0
+        self._connect_retry_cnt = 0
         self.connected = False
+        self._last_io_time = time.time()
 
     async def async_on_connected(self):
         """Socket connected notifications"""
@@ -43,14 +72,14 @@ class WpSocketClient:
         """Socket connect"""
         await self._async_wait_for_disconnect()
 
-        _LOGGER.debug('connecting to server %s', self._host)
+        _LOGGER.debug('connecting to server %s', self.host)
         try:
             asyncio.set_event_loop(self._loop)
             self._reader, self._writer = await asyncio.wait_for(
-                asyncio.open_connection(self._host, self._port),
+                asyncio.open_connection(self.host, self.port),
                 timeout=_CONNECTION_TIMEOUT)
             self.connected = True
-            self._retry_cnt = 0
+            self._connect_retry_cnt = 0
 
             self._loop.create_task(self.async_on_connected())
 
@@ -60,23 +89,25 @@ class WpSocketClient:
             self._wait_tasks = asyncio.wait(tasks)
             return True
         except Exception as ex:
-            _LOGGER.error("connection error, %s, %s", self._host, ex)
+            _LOGGER.error("connection error, %s, %s", self.host, ex)
             self.disconnect()
         return False
 
     async def _async_reconnect(self):
         """Socket reconnect"""
         self._loop.create_task(self.async_on_reconnect())
-        wait_time = self._retry_cnt * 5 if self._retry_cnt < 12 else 60
+        wait_time = self._connect_retry_cnt * 5 if self._connect_retry_cnt < 12 else 60
         _LOGGER.debug('reconnect connect, wait %d s', wait_time)
         await asyncio.sleep(wait_time)
-        self._retry_cnt += 1
+        self._connect_retry_cnt += 1
         if not await self.async_connect():
             await self._async_reconnect()
 
-    async def async_send_packet(self, packet: bytes):
+    async def async_send_packet(self, packet: bytes) -> bool:
         """Send packet"""
-        await self._send_packet_queue.put(packet)
+        send_data = _WpSendData(packet)
+        await self._send_packet_queue.put(send_data)
+        return await send_data.wait()
 
     async def _async_reader_handler(self):
         """Queue read handler"""
@@ -99,11 +130,11 @@ class WpSocketClient:
         _LOGGER.debug('reader start')
         while self.connected:
             try:
-                data = await asyncio.wait_for(self._reader.read(_DEFAULT_READ_BUFFER),
-                                              timeout=_READ_TIMEOUT)
-                if not data:
+                if (data := await asyncio.wait_for(self._reader.read(_DEFAULT_READ_BUFFER),
+                                                   timeout=_READ_TIMEOUT)) is None:
                     self._loop.create_task(self._async_reconnect())
                     break
+                self._last_io_time = time.time()
 
                 _LOGGER.debug('Received [%d]: %s', len(data), data.hex())
                 await self._receive_packet_queue.put(data)
@@ -118,12 +149,28 @@ class WpSocketClient:
         _LOGGER.debug('writer start')
         while self.connected:
             try:
-                packet = await asyncio.wait_for(self._send_packet_queue.get(),
-                                                timeout=_QUEUE_WAIT_TIMEOUT)
-                self._writer.write(packet)
-                await self._writer.drain()
-                _LOGGER.debug('Send [%d]: %s', len(packet), packet.hex())
-                self._send_packet_queue.task_done()
+                send_data: _WpSendData = await asyncio.wait_for(self._send_packet_queue.get(),
+                                                                timeout=_QUEUE_WAIT_TIMEOUT)
+                while True:
+                    interval = time.time() - self._last_io_time
+                    if interval > _READ_WRITE_INTERVAL:
+                        self._writer.write(send_data.packet)
+                        await self._writer.drain()
+                        self._last_io_time = time.time()
+                        _LOGGER.debug('Send: %s', send_data.packet.hex())
+                        send_data.set(True)
+                        self._send_packet_queue.task_done()
+                        break
+
+                    if send_data.inc_try() > _WRITE_RETRY_COUNT:
+                        send_data.set(False)
+                        _LOGGER.warning('Send fail: %s', send_data.packet.hex())
+                        break
+
+                    delay = _READ_WRITE_INTERVAL - interval
+                    _LOGGER.info('Send wait: %f', delay)
+                    await asyncio.sleep(delay)
+
             except asyncio.TimeoutError:
                 continue
             except Exception as ex:
